#!/usr/bin/env python3

import smbus
import time
import paho.mqtt.client as mqt
from datetime import datetime
from datetime import timedelta

bus = smbus.SMBus(1)
mqtc = mqt.Client()




access = {}
# [ code : [last_access, penalty] ]


def allowed(code):
    print (access)
    if code in access:
        if millis_since(access[code][0]) > 3000+access[code][1]:
            access[code] = [datetime.now(),0]
            return True
        else:
            access[code] = [datetime.now(),access[code][1]+3000]
            return False
    else:
        access[code] = [datetime.now(),0]
        return True

def millis_since(start_time):
    dt = datetime.now() - start_time
    ms = (dt.days * 24 * 60 * 60 + dt.seconds) * 1000 + dt.microseconds / 1000.0
    return ms



# Settings for connection
host = "10.42.0.244"
topic= "foobar/oben/licht"
port = 1883

# Callbacks
rc = 0

def on_connect(mosq, obj, foo ,bar ):
    print( "Connected" )

def on_message(mosq, obj, msg):
    #print( "Received on topic: " + msg.topic + " Message: "+str(msg.payload) );
    msgs(msg.payload, msg.topic)

def on_subscribe(mosq, obj, mid, granted_qos):
    print("Subscribed OK")

# Set callbacks
mqtc.on_message = on_message
mqtc.on_connect = on_connect
mqtc.on_subscribe = on_subscribe

# Connect and subscribe

def init_mqtt():
    mqtc.connect(host, port, 60)
    mqtc.subscribe(topic, 0)
    rc  = 0
    while rc == 0:
        rc = mqtc.loop()



#Funktion Setze Bit in Variable / Function Set Bit in byte
def set_bit(value, bit):
    return value | (1<<bit)

#Funktion rücksetzte Bit in Variable / Function reset Bit in byte
def clear_bit(value, bit):
    return value & ~(1<<bit)


#Buttonbefehle
def switch(i,speed=0.5):
    if allowed(i):
        o = 0
        if i > 7:
            o = set_bit(o,i-8)
            bus.write_byte(0x21,255-o)
            time.sleep(speed)
            o = clear_bit(o,i-8)
            bus.write_byte(0x21,255-o)
        else:
            o = set_bit(o, i)
            bus.write_byte(0x3f,255-o)
            time.sleep(speed)
            o = clear_bit(o,i)
            bus.write_byte(0x3f,255-o)
    

commands = { "flur" : 0, "baellebad" : 1, "lounge-front": 2, "lounge-back" : 3,"baellebad-ein" : 4, "lounge-ein" : 5, "cantina-ein" : 6, "zentral-aus" : 7, "cantina" : 8}
power = { "zentral-aus" : 7 , "baellebad" : 4, "lounge" : 5, "cantina" : 6 }
light = { "flur" : 0, "baellebad" : 1, "lounge-front" : 2, "lounge-back" : 3, "cantina" : 8 }
state  = []

def decode_topic(topic):
    #lazy
    parts = topic.explode("/")
    if parts[0] == "foobar" and parts[4] == "action":
        if parts[2] == "licht":
            return (parts[3],lights)
        elif parts[2] == "power":
            return (parts[3],power)
    return ("err", [] )

def msgs(inp, topic):
    c = inp.decode("utf-8")
    l = len(c)
    # supporting number commands
    if c == "OFF" or c == "ON":
        (name,lookup) = decode_topic(topic)
        if name == "err":
            return
        switch(lookup[name])
        if i in state:
            if state[i] == 0:
                state[i] = 1
            else:
                state[i] = 0
        else:
            state[i] = 1
        #later: send status based on state[i]
        
    elif l < 3:
        try:
            msg = int(inp)
            switch(msg)
        except ValueError:
            print ( "No valid value")
            return

    #supporting string commmands
    else:

        cmds = c.split(",")
        print ( "Command:", cmds[0], "Number of Parameters:", len(cmds) )

        #suporting string commands with an arg separated by ','
        if len(cmds) > 1:
            
            #error checking
            try:
                command = commands[cmds[0]]
            except KeyError:
                return
            try:
                arg = int (cmds[1])
            except ValueError:
                return
            
            #strobo
            if ( command == 100 or command == 99 ):
                if arg < 100:
                    for i in range(arg):
                        switch((100-command)*8, speed=0.05)
                        time.sleep(0.06)
            
            #command with parameter used for dimming
            else:       
                switch(command,speed = 4*arg/100+1 )

        #single string command without parameter
        else:
            try:
                switch(commands[c])
            except KeyError:
                return
        
    return


init_mqtt()
while 1:
    time.sleep(10)

# vim: noai:ts=4:sw=4
